'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _bluebirdSettle = require('bluebird-settle');

var _bluebirdSettle2 = _interopRequireDefault(_bluebirdSettle);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 MigratingAdapter
 Migrates files stored in other FileAdapter(s) to a new FileAdapter.
 All new writes will write to the main adapter.
 All reads will first read from the main adapter. If the file is not there,
 it will read from the old adapters and then store the results in the main adapter.
 */
class MigratingAdapter {

  /**
   * @param  {[FileAdapter]} mainAdapter The new adapter to use
   * @param  {[array]} oldAdapters The previous adapters
   */
  constructor(mainAdapter, oldAdapters) {
    this.mainAdapter = mainAdapter;
    this.oldAdapters = oldAdapters;

    if (!mainAdapter) {
      throw new Error('Main adapter required');
    }

    if (!oldAdapters || oldAdapters.length === 0) {
      throw new Error('At least one old adapter is required');
    }
  }

  // For a given config object, filename, and data, store a file
  // Returns a promise
  createFile(filename, data) {
    return this.mainAdapter.createFile(filename, data);
  }

  deleteFile(filename) {
    // Attempt a delete from all stores:
    const adapters = [this.mainAdapter].concat(this.oldAdapters);
    const promises = adapters.map(adapter => {
      return _bluebirdSettle2.default.resolve(adapter.deleteFile(filename));
    });

    return _bluebirdSettle2.default.settle(promises).then(results => {
      for (const result of results) {
        if (result.isFulfilled()) {
          return _bluebirdSettle2.default.resolve(result.value());
        }
      }

      // None were resolved
      return _bluebirdSettle2.default.reject(results[0].reason());
    });
  }

  getFileData(filename) {
    return this.mainAdapter.getFileData(filename).catch(err => {
      const promises = this.oldAdapters.map(adapter => {
        return adapter.getFileData(filename);
      });

      return _bluebirdSettle2.default.any(promises).then(result => {
        this.createFile(filename, result);
        return result;
      }).catch(() => {
        return err;
      });
    });
  }

  getFileLocation(config, filename) {
    return this.mainAdapter.getFileLocation(config, filename);
  }

  // TODO: Handle file streams
  // getFileStream(filename) { }
}
exports.default = MigratingAdapter;
module.exports = exports['default'];